use crate::{
    change::{ApplyChange, Change},
    marks::{Mark, MarkSet},
};

/// The state of an editor with respect to a particular buffer.
///
/// Contains all state that's unique to a buffer-editor pair.
#[derive(Debug, Clone)]
pub struct EditorState {
    pub view_position: Mark,
    pub cursors: MarkSet,
}

impl EditorState {
    pub fn new() -> Self {
        let mut cursors = MarkSet::new();
        cursors.add_mark(Mark::new(0, 0));

        EditorState {
            view_position: Mark::new(0, 0),
            cursors: cursors,
        }
    }
}

impl ApplyChange for EditorState {
    fn apply_change(&mut self, change: &Change) {
        self.view_position.apply_change(change);
        self.cursors.apply_change(change);
    }

    fn apply_change_inverse(&mut self, change: &Change) {
        self.view_position.apply_change_inverse(change);
        self.cursors.apply_change_inverse(change);
    }
}
