use crate::change::Change;

#[derive(Debug, Clone)]
pub struct History {
    edits: Vec<Change>,
    position: usize, // Where we are in the history.
}

impl History {
    pub fn new() -> History {
        History {
            edits: Vec::new(),
            position: 0,
        }
    }

    pub fn push_edit(&mut self, edit: Change) {
        self.commit();
        self.edits.push(edit);
        self.position += 1;
    }

    pub fn undo(&mut self) -> Option<&Change> {
        if self.position > 0 {
            self.position -= 1;
            Some(&self.edits[self.position])
        } else {
            None
        }
    }

    pub fn redo(&mut self) -> Option<&Change> {
        if self.position < self.edits.len() {
            let edit = &self.edits[self.position];
            self.position += 1;
            Some(edit)
        } else {
            None
        }
    }

    /// Takes the current undo state, and pushes that as another change on
    /// top of the existing undo stack, without deleting any existing items.
    pub fn commit(&mut self) {
        if self.position == self.edits.len() {
            return;
        }

        let mut undo_trans = Change::new();
        for trans in self.edits[self.position..].iter().rev() {
            undo_trans = undo_trans.compose(&trans.inverse());
        }
        self.edits.push(undo_trans);
        self.position = self.edits.len();
    }
}
